
# Wiki -- a part of The Ming Wiki

# Copyright (C) 2006 Billy-Bob Ming
# billy-bob@billy-bob-ming.com
# Released under the GNU General Public License
# (See the included COPYING file)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

# Hack History
# Copyright (C) 1999, 2000 Martin Pool <mbp@humbug.org.au> - piki

import cgi, sys, re
import cgitb; cgitb.enable()
from os import path, environ

import LibWiki, Page

class Wiki:

	INCOMING_MAX = 8*1024                  # max size allowed in incoming data
	word_re_str = r"\b([A-Z][a-z]+){2,}\b" #  defines WikiWord
	word_anchored_re = re.compile('^' + word_re_str + '$')

	def __init__(self, data, css, logo):
		self.data = data
		self.css = css
		self.logo = logo
		return

	#
	#  MAIN
	#

	def start(self):
		form = cgi.FieldStorage()
		print "Content-type: text/html\n"

		# tmp security
		remote_name = environ.get('REMOTE_ADDR', '')
		if (not (remote_name.find("63.229") == 0) and
			(not path.exists(path.expanduser("~/.my_machine")))):
			print '<h1>&nbsp;</h1>'
			print '<h3 align="center">403 -- Access Forbidden</h3>'
			print '<p align="center">You do not have permission to access this resource.'
			raise SystemExit
		# tmp security

		self.lib = LibWiki.LibWiki(self.data, self.logo, self.css, form, self.word_anchored_re)

		handlers = { 'fullsearch':	self._do_fullsearch,
					 'titlesearch': self._do_titlesearch,
					 'edit':		self._do_edit_1,
					 'access':      self._do_edit_2,
					 'savepage':	self._do_savepage }

		for cmd in handlers.keys():
			if form.has_key(cmd):
				if form.has_key('savetext'):
					apply(handlers[cmd], (form[cmd].value, form['savetext'].value,))
				elif form.has_key('access'):
					apply(handlers[cmd],
						  (form[cmd].value, form['name'].value, form['password'].value,))
				else:
					apply(handlers[cmd], (form[cmd].value,))
				break
		else:
			path_info = environ.get('PATH_INFO', '')

			if len(path_info) and path_info[0] == '/':
				query = path_info[1:] or 'FrontPage'
			else:		
				query = environ.get('QUERY_STRING', '') or 'FrontPage'

			word_match = re.match(self.word_re_str, query)
			if word_match:
				word = word_match.group(0)
				p = Page.Page(self.lib, word)
				p.send_page()
			else:
				print "<p>Can't work out query \"<pre>" + query + "</pre>\""

		sys.stdout.flush()

	#
	# Private Handlers
	#

	def _do_fullsearch(self, needle):
		self.lib.send_title('Full text search for "%s"' % (needle))
		needle_re = re.compile(needle, re.IGNORECASE)
		hits = []
		all_pages = self.lib.page_list()
		for page_name in all_pages:
			body = self.lib.read_body(page_name)
			count = len(needle_re.findall(body))
			if count:
				hits.append((count, page_name))

		# The default comparison for tuples compares elements in order,
		# so this sorts by number of hits
		hits.sort()
		hits.reverse()

		print "<UL>"
		for (count, page_name) in hits:
			print '<LI>' + self.lib.link_to(page_name)
			print ' . . . . ' + `count`
			print ['match', 'matches'][count <> 1]
		print "</UL>"
		self.lib.print_search_stats(len(hits), len(all_pages))

	def _do_titlesearch(self, needle):
		# XXX: check needle is legal -- probably can accept any RE
		self.lib.send_title("Title search for \"" + needle + '"')
		needle_re = re.compile(needle, re.IGNORECASE)
		all_pages = self.lib.page_list()
		hits = filter(needle_re.search, all_pages)
		print "<UL>"
		for filename in hits:
			print '<LI>' + self.lib.link_to(filename)
		print "</UL>"
		self.lib.print_search_stats(len(hits), len(all_pages))

	def _do_edit_1(self, pagename):
		if self.lib.is_public(pagename):
			self._do_edit(pagename)
		else:
			p = Page.Page(self.lib, pagename)
			p.send_authenticate()

	def _do_edit_2(self, pagename, usr, pwd):
		if self.lib.has_access(usr, pwd):
			self._do_edit(pagename)
		else:
			p = Page.Page(self.lib, pagename)
			str = "<b>You do not have edit access to this page.</b>"
			p.send_page(msg=str)
			
	def _do_edit(self, pagename):	
		p = Page.Page(self.lib, pagename)
		p.send_editor()

	def _do_savepage(self, pagename, saveval):
		pg = Page.Page(self.lib, pagename)
		if len(saveval) > self.INCOMING_MAX:
			str = """<b>Your edit exceeds the allowable size.</b>"""
		else:
			pg.save_text(saveval)
			str = """<b>Thank you for your changes.
			Your attention to detail is appreciated.</b>"""
		pg.send_page(msg=str)





# LibWiki -- a part of The Ming Wiki

# Copyright (C) 2006 Billy-Bob Ming
# billy-bob@billy-bob-ming.com
# Released under the GNU General Public License
# (See the included COPYING file)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

# Hack History
# Copyright (C) 1999, 2000 Martin Pool <mbp@humbug.org.au> - piki

import errno, string, time, os, stat
from os import path, environ
from time import localtime, strftime
if os.name == "nt":
	import msvcrt, sys
	Lib3._lock_sh = Lib3._lock_ex = Lib3._lock_win
	msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
else:
	import fcntl

class LibWiki:

	BOM_UTF8 = "\xef\xbb\xbf"
	BOM_UTF16_LE = "\xff\xfe"
	BOM_UTF16_BE = "\xfe\xff"
	datetime_fmt = '%a %d %b %Y %I:%M %p'
	nonexist_qm = 0	# show '?' for nonexistent?

	def __init__(self, data, logo, css_url, form, word_anchored_re):
		self.text_dir = path.join(data, 'text')
		self.editlog = path.join(data, 'editlog')
		self.access_txt = path.join(data, 'access.txt')
		self.public_txt = path.join(data, 'public.txt')
		self.logo_string = logo
		self.css_url = css_url
		self.form = form
		self.word_anchored_re = word_anchored_re
		return

	#
	# Public Methods
	#

	def editlog_add(self, page_name, host):
		editlog = self.lockopen(self.editlog, 'a+')
		try: 
			editlog.seek(0, 2)	# to end
			editlog.write(string.join((page_name, host, `time.time()`), "\t") + "\n")
		finally:
			editlog.close()

	def editlog_raw_lines(self):
		editlog = self.open(self.editlog, 'rt')
		try:
			return editlog.readlines()
		finally:
			editlog.close()

	def get_scriptname(self):
		return environ.get('SCRIPT_NAME', '')

	def has_access(self, usr, pwd):
		access = self.open(self.access_txt).readlines()
		if usr+":"+pwd+"\n" in access:
			return 1
		return 0

	def is_public(self, pagename):
		public = self.open(self.public_txt).readlines()
		if pagename+"\n" in public:
			return 1
		return 0

	def last_modified(self, page_name):
		fname = self._text_filename(page_name)
		if path.exists(fname):
			modtime = localtime(os.stat(fname)[stat.ST_MTIME])
			return strftime(self.datetime_fmt, modtime)
		else:
			return None

	def link_to(self, page_name):
		if path.exists(path.join(self.text_dir, page_name)):
			return self._link_tag(page_name)
		else:
			if self.nonexist_qm:
				return self._link_tag(page_name, '?', 'nonexistent') + page_name
			else:
				return self._link_tag(page_name, page_name, 'nonexistent')

	def page_list(self):
		return filter(self.word_anchored_re.match, os.listdir(self.text_dir))

	def print_footer(self, name, editable=1, mod_string=None):
		base = self.get_scriptname()
		print '<hr>'
		if editable:
			print self._link_tag('?edit='+name, 'EditText')
			print "of this page"
			if mod_string:
				print "(last modified %s)" % mod_string
			print '<br>'
		print self._link_tag('FindPage?value='+name, 'FindPage')
		print " by browsing, searching, or an index"

	def print_search_stats(self, hits, searched):
		print "<p>%d hits " % hits
		print " out of %d pages searched." % searched

	def read_body(self, page_name):
		try:
			return self.open(self._text_filename(page_name), 'rt').read()
		except IOError, er:
			if er.errno == errno.ENOENT:
				# just doesn't exist, use default
				return 'Describe %s here.' % page_name
			else:
				raise er

	def send_title(self, text, link=None, msg=None):
		print "<head><title>%s</title>" % text
		if self.css_url:
			print '<link rel="stylesheet" type="text/css" href="%s">' % self.css_url
		print "</head>"
		print '<body><h1>'
		if self.logo_string:
			print self._link_tag('RecentChanges', self.logo_string)
		if link:
			print '<a href="%s">%s</a>' % (link, text)
		else:
			print text
		print '</h1>'
		if msg: print msg
		print '<hr>'

	def write_file(self, page_name, text):
		tmp_filename = self._tmp_filename(page_name)
		self.lockopen(tmp_filename, 'wt').write(text)
		text = self._text_filename(page_name)
		if os.name == 'nt':
			try:
				os.remove(text)
			except OSError, er:
				if er.errno <> errno.ENOENT: raise er
		# XXX: POSIX rename ought to replace.
		os.rename(tmp_filename, text)

	def open(self, fname, *args):
		"""open() wrapper: if the file is opened for reading, we look
		for a possible BOM and (if we do find one) print some headache-saving
		info about it, moving to the beginning of the file's actual content
		"""
		f = open(fname, *args)
		if hasattr(f, "read"):
			bytes = f.read(2)
			if bytes in (self.BOM_UTF16_LE, self.BOM_UTF16_BE):
				raise IOError, (42, os.strerror(42), bytes)
			elif (bytes + f.read(1)) == self.BOM_UTF8:
				pass
			else:
				f.seek(0)
		return f

	def lockopen(self, fname, mode="r", create=True, block=False, retries=3, shared=-1):
		"""
		opens a file and applies a lock to it. By default, the lock
		will be a shared one if the file is opened for reading ONLY; exclusive
		in all other cases. This behaviour can be overridden by setting the 'shared'
		parameter to the correct boolean value. NOTES:
		- 'mode' must be a string following the same rules valid for the built-in
		  'file' function... but: append mode is not recognized, and the 'b' flag
		  has no effect (files are automatically opened in binary mode);
		- locking is *advisory* on Unix, mandatory on Windows;
		- on Windows, the lock will always be strictly exclusive.
		  "Strictly" means that the current process will not be able to map
		  another file descriptor to the locked file (and use it) until this
		  gets unlocked.
		"""
		if mode[0] not in ("a", "r", "w"):
			raise ValueError, "invalid mode " + mode
		EX = True
		if mode.find("+") != -1:
			FLAGS = os.O_RDWR
		elif (mode[0] == "w") or (mode[0] == "a"):
			FLAGS = os.O_WRONLY
		else:
			FLAGS = os.O_RDONLY
			EX = False
		if create:	FLAGS |= os.O_CREAT
		FLAGS |= getattr(os, "O_BINARY", 0)
		fd = os.open(fname, FLAGS)
		if EX == shared: EX ^= 1
		lock = EX and self._lock_ex or self._lock_sh
		for i in range(retries):
			try:
				lock(fd, block)
				fobj = os.fdopen(fd, mode)
				if mode[0] == "w": fobj.truncate()
				return fobj
			except Exception:
				time.sleep(.01)
		else:
			raise IOError, "cannot lock and open " + fname

	#
	# Private Methods
	#		

	def _link_tag(self, params, text=None, ss_class=None):
		if text is None:
			text = params					# default
		if ss_class:
			classattr = 'class="%s" ' % ss_class
		else:
			classattr = ''
		return '<a %s href="%s/%s">%s</a>' % \
			   (classattr, self.get_scriptname(), params, text)

	def _lock_nix(self, mode, fd, block):
		if not block:
			mode |= fcntl.LOCK_NB
		fcntl.lockf(fd, mode)

	def _lock_win(self, fd, block):
		MODE = block and msvcrt.LK_LOCK or msvcrt.LK_NBLCK
		msvcrt.locking(fd, MODE, 1)

	def _lock_sh(self, *args):
		"apply a shared lock"
		self._lock_nix(fcntl.LOCK_SH, *args)

	def _lock_ex(self, *args):
		"apply an exclusive lock"
		self._lock_nix(fcntl.LOCK_EX, *args)

	def _text_filename(self, page_name):
		return path.join(self.text_dir, page_name)

	def _tmp_filename(self, page_name):
		return path.join(self.text_dir, ('#' + page_name + '.' + `os.getpid()` + '#'))




# Page -- a part of The Ming Wiki

# Copyright (C) 2006 Billy-Bob Ming
# billy-bob@billy-bob-ming.com
# Released under the GNU General Public License
# (See the included COPYING file)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

# Hack History
# Copyright (C) 1999, 2000 Martin Pool <mbp@humbug.org.au> - piki

import string, re
from os import environ

import PageFormatter

class Page:

	def __init__(self, lib, page_name):
		self.pf = PageFormatter.PageFormatter(lib)
		self.page_name = page_name
		self.lib = lib

	#
	# Public Methods
	#

	def send_authenticate(self, msg=None):
		self.lib.send_title('Access ' + self._split_title())
		print '''<h3>Edit Access Requires Authentication</h3>
<form>
<h4>Username</h4>
<p><input type="text" maxlength="128" name="name" size="48" value="">
<h4>Password</h4>
<p><input type="password" maxlength="128" name="password" size="48" value="">
<h4>Submit</h4>
<input type="hidden" name="access" value="%s">
<input type="submit" name="submit" value=" Submit ">
</form>
''' % (self.page_name)

	def send_page(self, msg=None):
		link = self.lib.get_scriptname() + '?fullsearch=' + self.page_name
		self.lib.send_title(self._split_title(), link, msg)
		body = self.lib.read_body(self.page_name)
		self.pf.print_html(body)
		self.lib.print_footer(self.page_name, 1, self.lib.last_modified(self.page_name))

	def send_editor(self):
		self.lib.send_title('Edit ' + self._split_title())
		print '''<h3>Pages are limited to about three manuscript
		pages in size (8kb).</h3>'''
		print '<form method="post" action="%s">' % (self.lib.get_scriptname())
		print '<input type=hidden name="savepage" value="%s">' % (self.page_name)
		body = self.lib.read_body(self.page_name).replace('\r\n', '\n')
		print """<textarea wrap="virtual" name="savetext" rows="17"
				 cols="80">%s</textarea>""" % body
		print """<br><input type=submit value="Save">
				 <input type=reset value="Reset"><br></form>"""
		print "<p>" + self.lib.link_to('EditingTips')

	def save_text(self, newtext):
		self.lib.write_file(self.page_name, newtext)
		remote_name = environ.get('REMOTE_ADDR', '')
		self.lib.editlog_add(self.page_name, remote_name)

	#
	# Private Methods
	#

	def _split_title(self):
		return re.sub('([a-z])([A-Z])', r'\1 \2', self.page_name)




# PageFormatter -- a part of The Ming Wiki

# Copyright (C) 2006 Billy-Bob Ming
# billy-bob@billy-bob-ming.com
# Released under the GNU General Public License
# (See the included COPYING file)

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

# Hack History
# Copyright (C) 1999, 2000 Martin Pool <mbp@humbug.org.au> - piki

import re, string, time
from cStringIO import StringIO
from socket import gethostbyaddr
from time import localtime, strftime

class PageFormatter:
	"""Object that turns Wiki markup into HTML.

	All formatting commands can be parsed one line at a time, though
	some state is carried over between lines.
	"""
	show_hosts = 0	# show hostnames?
	changed_time_fmt = ' . . . . [%I:%M %p]'
	date_fmt = '%a %d %b %Y'
	
	def __init__(self, lib):
		self.form = lib.form
		self.lib = lib
		self.is_em = self.is_b = 0
		self.list_indents = []
		self.in_pre = 0

	def print_html(self, raw):
		self.raw = raw
		# For each line, we scan through looking for magic
		# strings, outputting verbatim any intervening text
		scan_re = re.compile(
			r"(?:(?P<emph>'{2,3})"
			+ r"|(?P<ent>[<>&])"
			+ r"|(?P<word>\b(?:[A-Z][a-z]+){2,}\b)"
			+ r"|(?P<rule>-{4,})"
			+ r"|(?P<url>(http|ftp|nntp|news|mailto)\:[^\s'\"]+\S)"
			+ r"|(?P<email>[-\w._+]+\@[\w.-]+)"
			+ r"|(?P<li>^\s+\*)"
			+ r"|(?P<pre>(\{\{\{|\}\}\}))"
			+ r"|(?P<macro>\[\[(TitleSearch|FullSearch|WordIndex"
							+ r"|TitleIndex|RecentChanges)\]\])"
			+ r")")
		blank_re = re.compile("^\s*$")
		bullet_re = re.compile("^\s+\*")
		indent_re = re.compile("^\s*")
		eol_re = re.compile(r'\r?\n')
		raw = string.expandtabs(self.raw)
		for line in eol_re.split(raw):
			if not self.in_pre:
				# XXX: Should we check these conditions in this order?
				if blank_re.match(line):
					print '<p>'
					continue
				indent = indent_re.match(line)
				print self._indent_to(len(indent.group(0)))
			print re.sub(scan_re, self._replace, line)
		if self.in_pre: print '</pre>'
		print self._undent()

	#
	# Private Methods
	#

	def _indent_level(self):
		return len(self.list_indents) and self.list_indents[-1]

	def _indent_to(self, new_level):
		s = ''
		while self._indent_level() > new_level:
			del(self.list_indents[-1])
			s = s + '</ul>\n'
		while self._indent_level() < new_level:
			self.list_indents.append(new_level)
			s = s + '<ul>\n'
		return s

	def _macro_repl(self, word):
		macro_name = word[2:-2]
		# XXX: Somehow get the default value into the search field
		return apply(getattr(self, '_macro_' + macro_name), ())		

	def _make_index_key(self):
		s = '<p><center>'
		links = map(lambda ch: '<a href="#%s">%s</a>' % (ch, ch), string.lowercase)
		s = s + string.join(links,' | ')
		s = s + '</center><p>'
		return s

	def _replace(self, match):
		for type, hit in match.groupdict().items():
			if hit:
				return apply(getattr(self, '_' + type + '_repl'), (hit,))
		else:
			raise "Can't handle match " + `match`

	def _undent(self):
		res = '</ul>' * len(self.list_indents)
		self.list_indents = []
		return res

	#
	# Repl Methods
	#

	def _email_repl(self, word):
		return '<a href="mailto:%s">%s</a>' % (word, word)

	def _emph_repl(self, word):
		if len(word) == 3:
			self.is_b = not self.is_b
			return ['</b>', '<b>'][self.is_b]
		else:
			self.is_em = not self.is_em
			return ['</em>', '<em>'][self.is_em]

	def _ent_repl(self, s):
		return {'&': '&amp;',
				'<': '&lt;',
				'>': '&gt;'}[s]

	def _li_repl(self, match):
		return '<li>'

	def _pre_repl(self, word):
		if word == '{{{' and not self.in_pre:
			self.in_pre = 1
			return '<pre>'
		elif self.in_pre:
			self.in_pre = 0
			return '</pre>'
		else:
			return ''

	def _rule_repl(self, word):
		s = self._undent()
		if len(word) <= 4:
			s = s + "\n<hr>\n"
		else:
			s = s + "\n<hr size=%d>\n" % (len(word) - 2 )
		return s

	def _url_repl(self, word):
		return '<a href="%s">%s</a>' % (word, word)

	def _word_repl(self, word):
		return self.lib.link_to(word)

	#
	# Macros 
	#

	def _macro_TitleSearch(self):
		return self._macro_search("titlesearch")

	def _macro_FullSearch(self):
		return self._macro_search("fullsearch")

	def _macro_search(self, type):
		if self.form.has_key('value'):
			default = self.form["value"].value
		else:
			default = ''
		return """<form method=get>
		<input name=%s size=30 value="%s"> 
		<input type=submit value="Go">
		</form>""" % (type, default)

	def _macro_WordIndex(self):
		s = self._make_index_key()
		pages = list(self.lib.page_list())
		map = {}
		word_re = re.compile('[A-Z][a-z]+')
		for name in pages:
			for word in word_re.findall(name):
				try:
					map[word].append(name)
				except KeyError:
					map[word] = [name]

		all_words = map.keys()
		all_words.sort()
		last_letter = None
		for word in all_words:
			letter = string.lower(word[0])
			if letter <> last_letter:
				s = s + '<a name="%s"><h3>%s</h3></a>' % (letter, letter)
				last_letter = letter

			s = s + '<b>%s</b><ul>' % word
			links = map[word]
			links.sort()
			last_page = None
			for name in links:
				if name == last_page: continue
				s = s + '<li>' + self.lib.link_to(name)
			s = s + '</ul>'
		return s

	def _macro_TitleIndex(self):
		s = self._make_index_key()
		pages = list(self.lib.page_list())
		pages.sort()
		current_letter = None
		for name in pages:
			letter = string.lower(name[0])
			if letter <> current_letter:
				s = s + '<a name="%s"><h3>%s</h3></a>' % (letter, letter)
				current_letter = letter
			else:
				s = s + '<br>'
			s = s + self.lib.link_to(name)
		return s

	def _macro_RecentChanges(self):
		lines = self.lib.editlog_raw_lines()
		lines.reverse()

		ratchet_day = None
		done_words = {}
		buf = StringIO()
		for line in lines:
			page_name, addr, ed_time = line.split('\t')
			# year, month, day, DoW
			time_tuple = localtime(float(ed_time))
			day = tuple(time_tuple[0:3])
			if day <> ratchet_day:
				buf.write('<h3>%s</h3>' % strftime(self.date_fmt, time_tuple))
				ratchet_day = day

			if done_words.has_key(page_name):
				continue

			done_words[page_name] = 1
			buf.write(self.lib.link_to(page_name))
			if self.show_hosts:
				buf.write(' . . . . ')
				try:
					buf.write(gethostbyaddr(addr)[0])
				except:
					buf.write("(unknown)")
			if self.changed_time_fmt:
				buf.write(time.strftime(self.changed_time_fmt, time_tuple))
			buf.write('<br>')

		return buf.getvalue()



#!/usr/bin/env python

# setup.py is part of The Ming Wiki
# billy-bob@billy-bob-ming.com

# Copyright (C) 2005, 2006	Richard Harris
# Released under the GNU General Public License
# (See the included COPYING file)

# The Ming Wiki is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# The Ming Wiki is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with The Ming Wiki; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA	 02110-1301	 USA


"""A Honking-Good Little Python Wiki

The Ming Wiki is compatible with but completely
independent of The Ming Server and The Ming Blogger.
"""

import sys, os, shutil
from distutils.cmd import Command
from distutils.core import setup
from distutils.command.install_data import install_data

NAME = "The Ming Wiki"
UNIX_NAME = "TheMingWiki"
LICENSE = "GNU General Public License"
PLATFORMS = ["any"]
CLASSIFIERS = """
Development Status :: 4 - Beta
Environment :: Web Environment
Intended Audience :: Developers
Intended Audience :: End Users/Desktop
Intended Audience :: System Administrators
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Operating System :: OS Independent
Programming Language :: Python
Topic :: Internet :: WWW/HTTP :: Dynamic Content
Topic :: Internet :: WWW/HTTP :: Site Management
"""

os.linesep = "\n"
get = globals().get

def walk_data(dir):
	result = []
	start = len(dir) + 1
	append = lambda foo, dir, names: result.append((dir[start:] or '.', \
					filter(os.path.isfile, [os.path.join(dir, name) for name in names])))
	os.path.walk(dir, append, None)
	return result

PACKAGES = [
		'TheMingWiki',
		]
SCRIPTS = ['cgibin/billiwik.cgi']
DATA_FILES = walk_data('data')
HTDOCS = walk_data('htdocs')

class install(Command):

	prompts = (
		('lib', ("Path to cgi-bin directory where The Ming Wiki will be installed", 0, 1)),
		('data', ("Path to your Ming Wiki data tree (ie /path/to/www/data)", 1, 1)),
		('htdocs', ("Path to your web server's htdocs directory (ie /path/to/public_html)", 1, 1))
		)

	def finalize_options(self):
		self.print_header()
		if not self.has_any_option_set():
			self.ask_user()
			self.install_scripts = self.install_lib

#=========================== the rest is constant for every package of Billy Bob's

		self.set_undefined_options('build',
								   ('build_lib', 'build_lib'))

	description = "install " + NAME

	user_options = [
		('install-lib=', None, "installation directory for Python modules"),
		('install-scripts=', None, "installation directory for Python scripts"),
		('install-data=', None, "installation directory for data files"),
		('install-htdocs=', None, "web server's htdocs directory (for static resources)"),
		('compile', 'c', "compile .py to .pyc [default]"),
		('no-compile', None, "don't compile .py files"),
		('optimize=', 'O',
		 "also compile with optimization: -O1 for \"python -O\", "
		 "-O2 for \"python -OO\", and -O0 to disable [default: -O0]"),
		('force', 'f',
		 "force installation (overwrite any existing files)")
		]

	def initialize_options(self):
		self.install_lib = None
		self.install_scripts = None
		self.install_data = None
		self.install_htdocs = None
		self.force = 0
		self.skip_build = 0
		self.build_lib = None
		self.compile = None
		self.optimize = None
		self.root = None

	def ask_user(self):
		i = 0
		for key, options in self.prompts:
			i += 1
			print "(%d)" % i,
			install_dir = self.get_dir(*options)
			setattr(self, "install_" + key, install_dir)

	def get_dir(self, prompt, optional=0, create=1):
		print "Enter %s%s:" % (prompt, optional and " (optional)" or "")
		while 1:
			try:
				install_dir = raw_input("> ").strip()
			except:
				print "\n\nInstallation interrupted by user."
				sys.exit()
			if optional and install_dir == "":
				return None
			elif os.path.exists(install_dir):
				break
			elif create:
				try:
					os.mkdir(install_dir)
					break
				except: pass
			print "Please specify a valid%s directory path%s..." % \
				(not create and " (and existing)" or "",
				 optional and ", or press Enter to skip the \ninstallation of this item" or "")
		print
		return install_dir

	def print_header(self):
		print "_" * 76
		print "\n  %s Installation Script" % NAME
		print "_" * 76
		print

	def run(self):
		commands = self.get_sub_commands()
		if commands:
			for cmd_name in commands:
				print "* installing %s..." % cmd_name[8:]
				self.run_command(cmd_name)
			print "\nInstallation completed."
		else:
			print "Nothing to do!"

	def has_lib(self):
		return self.install_lib and self.distribution.packages

	def has_scripts(self):
		return self.install_scripts and self.distribution.scripts

	def has_data(self):
		return self.install_data and self.distribution.data_files

	def has_htdocs(self):
		return self.install_htdocs and get('HTDOCS')

	def has_any_option_set(self):
		if (self.install_lib or
			self.install_data or
			self.install_scripts or
			self.install_htdocs):
			   return True
		return False

	sub_commands = [
		('install_lib',		has_lib),
		('install_scripts', has_scripts),
		('install_data',	has_data),
		('install_htdocs',	has_htdocs)
		]

class install_htdocs(install_data):

	description = "install static web resources and example files"

	def initialize_options(self):
		install_data.initialize_options(self)
		self.data_files = get('HTDOCS')

	def finalize_options (self):
		self.set_undefined_options('install',
								   ('install_htdocs', 'install_dir'),
								   ('root', 'root'),
								   ('force', 'force'),
								   )

#-------------------------------------------------------

if sys.version < '2.2.3':
	from distutils.dist import DistributionMetadata
	DistributionMetadata.classifiers = None
	DistributionMetadata.download_url = None

doclines = __doc__.splitlines()

setup(
	verbose = 0,
	cmdclass = {'install': install,
				'install_htdocs': install_htdocs},
	name = UNIX_NAME,
	version = "1.0-pre02",
	license = LICENSE,
	platforms = PLATFORMS,
	classifiers = filter(None, CLASSIFIERS.splitlines()),
	author = "Billy-Bob Ming",
	author_email = "billy-bob@billy-bob-ming.com",
	description = "A Honking-Good Little Python Wiki",
	url = "http://python.org/pypi/%s" % UNIX_NAME,
	long_description = "\n".join(doclines[2:]),
	package_dir = get('PACKAGE_DIR'),
	packages = get('PACKAGES'),
	scripts = get('SCRIPTS'),
	data_files = get('DATA_FILES')
	)
